use aoc_runner_derive::{aoc, aoc_generator};
use nom::character::complete::{newline, space0};
use nom::multi::{many0, many1};
use nom::{character::complete::i64 as parse_usize, IResult};

#[aoc_generator(day2)]
pub fn generator(input: &str) -> Vec<Vec<isize>> {
    many1(parse_line)(input).unwrap().1
}

fn parse_level(input: &str) -> IResult<&str, isize> {
    let (input, res) = parse_usize(input)?;
    let (input, _) = space0(input)?;
    Ok((input, res as isize))
}

fn parse_line(input: &str) -> IResult<&str, Vec<isize>> {
    let (input, levels) = many1(parse_level)(input)?;
    let (input, _) = many0(newline)(input)?;
    Ok((input, levels))
}

#[aoc(day2, part1)]
pub fn part1(input: &[Vec<isize>]) -> usize {
    input.iter().filter(|vec| is_valid(vec)).count()
}

fn is_valid(input: &[isize]) -> bool {
    let mut direction = 0;
    for i in 0..(input.len() - 1) {
        let diff = input[i + 1] - input[i];
        if diff == 0 {
            return false;
        } else if (1..=3).contains(&diff) {
            if direction == -1 {
                return false;
            }
            direction = 1;
        } else if (1..=3).contains(&-diff) {
            if direction == 1 {
                return false;
            }
            direction = -1;
        } else {
            return false;
        }
    }
    true
}

#[aoc(day2, part2)]
pub fn part2(input: &[Vec<isize>]) -> usize {
    input
        .iter()
        .filter(|rec| (0..rec.len()).any(|i| is_valid2(rec, i)))
        .count()
}

fn is_valid2(input: &[isize], skip: usize) -> bool {
    let mut direction = 0;
    for i in 0..(input.len() - 2) {
        let diff = if skip <= i {
            input[i + 2] - input[i + 1]
        } else if skip == i + 1 {
            input[i + 2] - input[i]
        } else {
            input[i + 1] - input[i]
        };
        if diff == 0 {
            return false;
        } else if (1..=3).contains(&diff) {
            if direction == -1 {
                return false;
            }
            direction = 1;
        } else if (1..=3).contains(&-diff) {
            if direction == 1 {
                return false;
            }
            direction = -1;
        } else {
            return false;
        }
    }
    true
}
